@@ -66,11 +66,13 @@ class DemoHeteroHost(HeteroNNHostScheduler):
                  feature_key: str,
                  batch_size: int,
                  data_dir: str,
+                 name: str = None,
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
                  log_rounds: int = 0) -> None:
         super().__init__(feature_key=feature_key,
+                         name=name,
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
@@ -200,9 +202,11 @@ class DemoHeteroCollaborator(HeteroNNCollaboratorScheduler):
                  feature_key: str,
                  batch_size: int,
                  data_dir: str,
+                 name: str = None,
                  schedule_timeout: int = 30,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
+                         name=name,
                          schedule_timeout=schedule_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
@@ -288,6 +292,7 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
                  project_layer_lr: float,
                  batch_size: int,
                  data_dir: str,
+                 name: str = None,
                  max_rounds: int = 0,
                  calculation_timeout: int = 300,
                  schedule_timeout: int = 30,
@@ -296,6 +301,7 @@ class DemoSecureHeteroHost(SecureHeteroNNHostScheduler):
         super().__init__(feature_key=feature_key,
                          project_layer_config=project_layer_config,
                          project_layer_lr=project_layer_lr,
+                         name=name,
                          max_rounds=max_rounds,
                          calculation_timeout=calculation_timeout,
                          schedule_timeout=schedule_timeout,
@@ -428,10 +434,12 @@ class DemoSecureHeteroCollaborator(SecureHeteroNNCollaboratorScheduler):
                  project_layer_lr: int,
                  batch_size: int,
                  data_dir: str,
+                 name: str = None,
                  schedule_timeout: int = 30,
                  is_feature_trainable: bool = True) -> None:
         super().__init__(feature_key=feature_key,
                          project_layer_lr=project_layer_lr,
+                         name=name,
                          schedule_timeout=schedule_timeout,
                          is_feature_trainable=is_feature_trainable)
         self.batch_size = batch_size
@@ -526,6 +534,7 @@ def get_host(mode: str = VANILLA) -> Union[DemoHeteroHost, DemoSecureHeteroHost]
         scheduler = DemoHeteroHost(feature_key='demo_host',
                                    batch_size=128,
                                    data_dir=_DATA_DIR,
+                                   name='demo_hetero_host',
                                    max_rounds=5,
                                    calculation_timeout=60,
                                    log_rounds=1)
@@ -540,6 +549,7 @@ def get_host(mode: str = VANILLA) -> Union[DemoHeteroHost, DemoSecureHeteroHost]
                                          project_layer_lr=0.01,
                                          batch_size=128,
                                          data_dir=_DATA_DIR,
+                                         name='demo_hetero_host',
                                          max_rounds=5,
                                          calculation_timeout=60,
                                          log_rounds=1)
@@ -564,14 +574,16 @@ def get_collaborator(mode: str = VANILLA) -> Union[DemoHeteroCollaborator,
     if mode == VANILLA:
         scheduler = DemoHeteroCollaborator(feature_key='demo_collaborator',
                                            batch_size=128,
-                                           data_dir=_DATA_DIR)
+                                           data_dir=_DATA_DIR,
+                                           name='demo_hetero_collaborator')
 
     else:
         # Too big batch size could kill the server
         scheduler = DemoSecureHeteroCollaborator(feature_key='demo_collaborator',
                                                  project_layer_lr=0.01,
                                                  batch_size=128,
-                                                 data_dir=_DATA_DIR)
+                                                 data_dir=_DATA_DIR,
+                                                 name='demo_hetero_collaborator')
 
     with open(pickle_file, 'w+b') as pf:
         pickle.dump(scheduler, pf)
