@@ -4,7 +4,7 @@ import secrets
 from dataclasses import dataclass
 from typing import List
 
-from ..contractor import (ApplySendingDataEvent, ContractEvent,
+from ..contractor import (ApplySharedFileSendingDataEvent, ContractEvent,
                           TaskMessageContractor)
 from .psi.rsa_psi_contractor import RSAContractor, RSAEventFactory
 
@@ -241,10 +241,23 @@ class CompleteTaskEvent(ContractEvent):
         return CompleteTaskEvent()
 
 
+@dataclass
+class ResetRoundEvent(ContractEvent):
+    """An event of resetting context of current training round."""
+
+    TYPE = 'reset_round'
+
+    @classmethod
+    def contract_to_event(cls, contract: dict) -> 'ResetRoundEvent':
+        event_type = contract.get('type')
+        assert event_type == cls.TYPE, f'合约类型错误: {event_type}'
+        return ResetRoundEvent()
+
+
 # define some more friendly alias
-SendFeatureEvent = ApplySendingDataEvent
-SendFeatureGradEvent = ApplySendingDataEvent
-SendTestFeatureEvent = ApplySendingDataEvent
+SendFeatureEvent = ApplySharedFileSendingDataEvent
+SendFeatureGradEvent = ApplySharedFileSendingDataEvent
+SendTestFeatureEvent = ApplySharedFileSendingDataEvent
 
 
 class HeteroNNEventFactory(RSAEventFactory):
@@ -262,6 +275,7 @@ class HeteroNNEventFactory(RSAEventFactory):
         CollaboratorCompleteEvent.TYPE: CollaboratorCompleteEvent,
         FailTaskEvent.TYPE: FailTaskEvent,
         CompleteTaskEvent.TYPE: CompleteTaskEvent,
+        ResetRoundEvent.TYPE: ResetRoundEvent,
         **RSAEventFactory._CLASS_MAP,
     }
 
@@ -287,7 +301,7 @@ class HeteroNNContractor(RSAContractor):
         self._new_contract(targets=self.EVERYONE, event=event)
         return nonce
 
-    def respond_checkin(self,
+    def respond_check_in(self,
                         round: int,
                         host: str,
                         nonce: str,
@@ -343,3 +357,8 @@ class HeteroNNContractor(RSAContractor):
         """Finish the specified task."""
         event = CompleteTaskEvent() if is_succ else FailTaskEvent()
         self._new_contract(targets=self.EVERYONE, event=event)
+
+    def reset_round(self):
+        """Reset current training round."""
+        event = ResetRoundEvent()
+        self._new_contract(targets=self.EVERYONE, event=event)
