@@ -7,7 +7,7 @@ from dataclasses import asdict, dataclass
 from typing import Dict, List, Optional, Set
 
 import gmpy2
-from Crypto.PublicKey import RSA
+from Cryptodome.PublicKey import RSA
 from Cryptodome import Random
 
 from ... import logger
