@@ -1,10 +1,12 @@
 """Algorithm scheduler."""
 
+
+from abc import ABC, abstractmethod
+from functools import wraps
 import inspect
 import os
 import sys
-from abc import ABC, abstractmethod
-from typing import Set, Tuple
+from typing import Dict, Iterable, Set, Tuple
 from zipfile import ZipFile
 
 import cloudpickle as pickle
@@ -12,6 +14,7 @@ import cloudpickle as pickle
 from . import logger, task_logger
 from .bass import BassProxy
 from .contractor import TaskContractor
+from .metrics_trace import MetricsTrace
 
 
 class ConfigError(Exception):
@@ -59,6 +62,10 @@ class DataChecker(ABC):
 
 class Scheduler(ABC):
 
+    def __init__(self) -> None:
+        super().__init__()
+        self._metrics_bucket: Dict[str, MetricsTrace] = {}
+
     @property
     def bass_proxy(self) -> BassProxy:
         if not hasattr(self, '_bass_proxy'):
@@ -119,6 +126,9 @@ class Scheduler(ABC):
         else:
             logger.warn('Failed to push a message because context is not initialized.')
 
+    def get_metrics(self, name: str) -> MetricsTrace:
+        return self._metrics_bucket.get(name)
+
     def _switch_status(self, _status: str):
         """Switch to a new status and leave a log."""
         self.status = _status
@@ -141,3 +151,43 @@ class Scheduler(ABC):
             recover:
                 whether to try recovering from last failed running
         """
+
+
+def register_metrics(name: str, keys: Iterable[str]):
+    """Register a metrics trace.
+
+    NOTE: Deprecated.
+
+    The trace fo the metrics during training will be included in the task's
+    running result and can be downloaded. However, the trace content should
+    be appended manually.
+
+    If the specified name has already registered before, nothing changes.
+
+    :args
+        :name
+            The name of the metrics trace.
+        :keys
+            The names of the metrics items.
+    """
+    def decorate(func):
+        import warnings
+        warnings.warn('`register_metrics` is deprecated and will be removed sooner',
+                      category=DeprecationWarning)
+        if not name or not isinstance(name, str):
+            raise ValueError(f'invalid name of the metrics: {name}')
+        if not keys or not isinstance(keys, Iterable):
+            raise ValueError(f'invalid keys of the metrics: {keys}')
+
+        @wraps(func)
+        def wrapper(_self, *args, **kwargs):
+            if not isinstance(_self, Scheduler) or func.__name__ != 'test':
+                raise RuntimeError(
+                    'Only applicable on "test" function of a "Scheduler" object.'
+                )
+            if name not in _self._metrics_bucket:
+                _self._metrics_bucket[name] = MetricsTrace(headers=keys)
+            return func(_self, *args, **kwargs)
+
+        return wrapper
+    return decorate
